;;;
;;; Definitions of the cairo context and other drawing realted things.
;;; See http://www.cairographics.org/manual/Drawing.html
;;;

(in-package :cffi-cairo)

(defcairoclass nil cr context-t
  (:constructors
   ("create" (:args (target surface-t))))
  (:methods
   ("destroy")
   ("status" (:return-type status-t))
   ("save")
   ("restore")
   ("get_target" (return-type surface-t))
   ("push_group")
   ("push_group_with_content" (:args (content content-t)))
   ("pop_group" (:return-type pattern-t))
   ("pop_group_to_source")
   ("get_group_target" (:return-type surface-t))
   ("set_source_rgb" (:args (red :double) (green :double) (blue :double)))
   ("set_source_rgba"
    (:args (red :double) (green :double) (blue :double) (alpha :double)))
   ("set_source" (:args (pattern pattern-t)))
   ("set_source_surface" (:args (surface surface-t) (x :double) (y :double)))
   ("get_source" (:return-type pattern-t))
   ("set_antialias" (:args (antialias antialias-t)))
   ("get_antialias" (:return-type antialias-t))
   ("set_dash" (:args (dashes :pointer) (num-dashes :int) (offset :double)))
   ("get_dash_count" (:return-type :int))
   ("get_dash" (:args (dashes :pointer) (offset :double)))
   ("set_fill_rule" (:args (fill-fule fill-rule-t)))
   ("get_fill_rule" (:return-type fill-rule-t))
   ("set_line_cap" (:args (line-cap line-cap-t)))
   ("get_line_cap" (:return-type line-cap-t))
   ("set_line_join" (:args (line-join line-join-t)))
   ("get_line_join" (:return-type line-join-t))
   ("set_line_width" (:args (width :double)))
   ("get_line_width" (:return-type :double))
   ("set_miter_limit" (:args (limit :double)))
   ("get_miter_limit" (:return-type :double))
   ("set_operator" (:args (op operator-t)))
   ("get_operator" (:return-type operator-t))
   ("set_tolerance" (:args (tolerance :double)))
   ("get_tolerance" (:return-type :double))
   ("clip")
   ("clip_preserve")
   ("clip_extents"
    (:args (x1 :pointer) (y1 :pointer) (x2 :pointer) (y2 :pointer))
    (:alternative 
     (:pointers (x1 :double) (y1 :double) (x2 :double) (y2 :double))
     (:body (list x1* y1* x2* y2*))))
   ("reset_clip")
   ("copy_clip_rectangle_list" (:return-type rectangle-list-pt))
   (("fill" fill-path))
   ("fill_preserve")
   ("fill_extents"
    (:args (x1 :pointer) (y1 :pointer) (x2 :pointer) (y2 :pointer))
    (:alternative 
     (:pointers (x1 :double) (y1 :double) (x2 :double) (y2 :double))
     (:body (list x1* y1* x2* y2*))))
   ("in_fill" (:return-type bool-t) (:args (x :double) (y :double)))
   ("mask" (:args (pattern pattern-t)))
   ("mask_surface"
    (:args (surface surface-t) (surface-x :double) (surface-y :double)))
   ("paint")
   ("paint_with_alpha" (:args (alpha :double)))
   ("stroke")
   ("stroke_preserve")
   ("stroke_extents"
    (:args (x1 :pointer) (y1 :pointer) (x2 :pointer) (y2 :pointer))
    (:alternative 
     (:pointers (x1 :double) (y1 :double) (x2 :double) (y2 :double))
     (:body (list x1* y1* x2* y2*))))
   ("in_stroke" (:return-type bool-t) (:args (x :double) (y :double)))
   ("copy_page")
   ("show_page")
   ("get_reference_count" (:return-type :uint))
   ("set_user_data"
    (:args (key user-data-key-t) (user-data :pointer)
	   (destroy destroy-func-t)))
   ("get_user_data" (:return-type :pointer) (:args (key user-data-key-t)))
   
   ("copy_path" (:return-type pattern-t))
   ("copy_path_flat" (:return-type pattern-t))
   ("append_path" (:args (path path-pt)))
   ("get_current_point"
    (:args (x :pointer) (y :pointer))
    (:alternative
     (:pointers (x :double) (y :double))
     (:body (list x y))))
   ("new_path")
   ("new_sub_path")
   ("close_path")
   ("arc" (:args (xc :double) (yc :double)
		 (radius :double)
		 (angle1 :double) (angle2 :double)))
   ("arc_negative" (:args (xc :double) (yc :double)
			  (radius :double)
			  (angle1 :double) (angle2 :double)))
   ("curve_to" (:args (x1 :double) (y1 :double)
		      (x2 :double) (y2 :double)
		      (x3 :double) (y3 :double)))
   ("line_to" (:args (x :double) (y :double)))
   ("move_to" (:args (x :double) (y :double)))
   ("rectangle" (:args (x :double) (y :double)
		       (width :double) (height :double)))
   ("glyph_path" (:args (glyphs :pointer) (num-glyphs :int)))
   ("text_path" (:args (utf8 :string)))
   ("rel_curve_to" (:args (dx1 :double) (dy1 :double)
			  (dx2 :double) (dy2 :double)
			  (dx3 :double) (dy3 :double)))
   ("rel_line_to" (:args (dx :double) (dy :double)))
   ("rel_move_to" (:args (dx :double) (dy :double)))
   ("translate" (:args (tx :double) (ty :double)))
   ("scale" (:args (sx :double) (sy :double)))
   ("rotate" (:args (angle :double)))
   ("transform" (:args (matrix matrix-pt)))
   ("set_matrix" (:args (matrix matrix-pt)))
   ("get_matrix" (:args (matrix matrix-pt)))
   ("identity_matrix")
   ("user_to_device"
    (:args  (x :pointer) (y :pointer))
    (:alternative (:args x y)
		  (:pointers (x :int) (y :int))
		  (:body (list x* y*))))
   ("user_to_device_distance"
    (:args (dx :pointer) (dy :pointer))
    (:alternative (:args dx dy)
		  (:pointers (dx :int) (dy :int))
		  (:body (list dx* dy*))))
   ("device_to_user"
    (:args (x :pointer) (y :pointer))
    (:alternative (:args x y)
		  (:pointers (x :int) (y :int))
		  (:body (list x* y*))))
   ("device_to_user_distance"
    (:args (dx :pointer) (dy :pointer))
    (:alternative (:args dx dy)
		  (:pointers (dx :int) (dy :int))
		  (:body (list dx* dy*))))
   ("select_font_face"
    (:args  (family :string) (slant font-slant-t)
	  (weight font-weight-t)))
   ("set_font_size" (:args (size :double)))
   ("set_font_matrix" (:args (matrix matrix-pt)))
   ("get_font_matrix" (:args (matrix matrix-pt)))
   ("set_font_options" (:args (font-options :pointer)))
   ("get_font_options" (:args (font-options :pointer)))
   ("set_font_face" (:args (font-face :pointer)))
   ("get_font_face" (:return-type :pointer))
   ("set_scaled_font" (:args (scaled-font :pointer)))
   ("get_scaled_font" (:return-type :pointer))
   ("show_text" (:args (utf8 :string)))
   ("show_glyphs" (:args (glyphs :pointer) (num_glyphs :int)))
   ("font_extents" (:args (font-extents :pointer)))
   ("text_extents" (:args (utf8 :pointer) (extents :pointer)))
   ("glyph_extents"
    (:args  (glyphs :pointer) (num-glyphs :int)
	    (extents :pointer)))))


(defcairoclass "path" path path-pt
  (:methods
   ("destroy")))

(defcairoclass "rectangle_list" rectangle-list rectangle-list-pt
  (:methods
   ("destroy")))