;;;
;;; Definitions of the things needed for fonts
;;; See http://www.cairographics.org/manual/Fonts.html
;;;

(in-package :cffi-cairo)

(defcairoclass "font_face" font-face font-face-t
  (:constructors
   ("reference" (:args (font-face font-face-t))))
  (:methods
   ("destroy")
   ("status" (:return-type status-t))
   ("get_type" (:return-type font-type-t))
   ("get_reference_count" (:return-type :uint))
   ("set_user_data"
    (:return-type status-t)
    (:args (key user-data-key-t) (user-data :pointer)
	   (destroy destroy-func-t)))
   ("get_user_data" (:return-type :pointer) (:args  (key user-data-key-t)))))

(defcairoclass "scaled_font" scaled-font scaled-font-t
  (:constructors
   ("create"
    (:args (font-face font-face-t) (font-matrix matrix-pt) (ctm matrix-pt)
	   (options font-options-t)))
   ("reference" (:args (scaled-font scaled-font-t))))
  (:methods
   ("destroy")
   ("status" (:return-type status-t))
   ("extents"
    (:args (extents :pointer))
    (:alternative (:pointers (extents 'font-extents-t))
		  (:body extents*)))
   ("text_extents"
    (:args (utf8 :string) (extents :pointer))
    (:alternative (:args utf8)
		  (:pointers (extents 'text-extents-t))
		  (:body extents*)))
   ("glyph_extents"
    (:args (glyphs glyph-pt) (num-glyphs :int) (extents :pointer))
    (:alternative (:args glyphs num-glyphs)
		  (:pointers (extents 'text-extents-t))
		  (:body extents*)))
   ("get_font_face" (:return-type font-face-t))
   ("get_font_options"
    (:args (options :pointer))
    (:alternative (:pointers (options 'font-options-t))
		  (:body options*)))))

(defcairoclass "font_options" options font-options-t
  (:constructors
   ("create")
   ("copy" (:args (original font-options-t))))
  (:methods
   ("destroy")
   ("status" (:return-type status-t))
   ("merge" (:args (other :pointer)))
   ("hash" (:return-type :ulong))
   ("equal" (:return-type bool-t) (:args (other font-options-t)))
   ("set_antialias" (:args (antialias antialias-t)))
   ("get_antialias" (:return-type antialias-t))
   ("set_subpixel_order" (:args  (subpixel-order subpixel-order-t)))
   ("get_subpixel_order" (:return-type subpixel-order-t))
   ("set_hint_style" (:args  (hint-style hint-style-t)))
   ("get_hint_style" (:return-type hint-style-t))
   ("set_hint_metrics" (:args (hint-metrics hint-metrics-t)))
   ("get_hint_metrics" (:return-type hint-metrics-t))))