;;;
;;; Definitions of the surfaces.
;;; See http://www.cairographics.org/manual/Surfaces.html
;;;

(in-package :cffi-cairo)

(defcairoclass "surface" surface surface-t
  (:constructors
   ("create_similar" (:args (other :pointer) (content content-t)))
   ("reference" (:args (surface surface-t))))
  (:methods
   ("destroy")
   ("status" (:return-type status-t))
   ("finish")
   ("flush")
   ("get_font_options" (options :pointer))
   ("get_content" (:return-type content-t))
   ("mark_dirty")
   ("mark_dirty_rectangle"
    (:args (x :int) (y :int) (width :int) (height :int)))
   ("set_device_offset" (:args (x-offset :double) (y-offset :double)))
   ("get_device_offset"
    (:args (x-offset :pointer) (y-offset :pointer))
    (:alternative (:pointers (x-offset :double) (y-offset :double))
		  (:body (list x-offset* y-offset*))))
   ("set_fallback_resolution"
    (:args (x-pixels-per-inch :double) (y-pixels-per-inch :double)))
   ("get_type" (:return-type surface-type-t))
   ("get_reference_count" (:return-type :uint))
   ("set_user_data"
    (:return-type status-t)
    (:args (key :pointer) (user-data :pointer) (destroy :pointer)))
   ("get_user_data" (:return-type :pointer) (:args (key :pointer)))
   ("write_to_png" (:return-type status-t) (:args (filename :string)))
   ("write_to_png_stream"
    (:return-type status-t)
    (:args (write-func :pointer) (closure :pointer)))))

(defcairoclass "image_surface" surface surface-t
  (:constructors
   ("create" (:args (format format-t) (width :int) (height :int)))
   ("create_for_data" 
    (:args (data :pointer) (format format-t)
	   (width :int) (height :int) (stride :int)))
   ("create_from_png" (:args (filename :string)))
   ("create_from_png_stream" (:args (read-func :pointer) (closure :pointer))))
  (:methods
   ("get_data" (:return-type :pointer))
   ("get_format" (:return-type format-t))
   ("get_width" (:return-type :int))
   ("get_height" (:return-type :int))
   ("get_stride" (:return-type :int))))

(defcairoclass "pdf_surface" surface surface-t
  (:constructors
   ("create"
    (:args (filename :string)
	   (width-in-points :double) (height-in-points :double)))
   ("create_for_stream"
    (:args (write-func :pointer) (closure :pointer)
	   (width-in-points :double) (height-in-points :double))))
  (:methods
   ("set_size"
    (:args (width-in-points :double) (height-in-points :double)))))

(defcairoclass "ps_surface" surface surface-t
  (:constructors
   ("create"
    (:args (filename :string)
	   (width-in-points :double) (height-in-points :double)))
   ("create_for_stream"
    (:args (write-func :pointer) (closure :pointer)
	   (width-in-points :double) (height-in-points :double))))
  (:methods
   ("set_size"
    (:args (width-in-points :double) (height-in-points :double)))
   ("dsc_begin_setup")
   ("dsc_begin_page_setup")
   ("dsc_comment" (:args (comment :string)))))

(defcairoclass "win32_surface" surface surface-t
  (:constructors
   ("create" (:args (hdc hdc-t)))
   ("create_with_dib" (:args (format format-t) (width :int) (height :int)))
   ("create_with_ddb"
    (:args (hdc hdc-t) (format format-t) (width :int) (height :int))))
  (:methods
   ("get_dc" (:return-type hdc-t))
   ("get_image" (:return-type surface-t))))

(defcairoclass "svg_surface" surface surface-t
  (:constructors
   ("create"
    (:args (filename :string)
	   (width-in-points :double) (height-in-points :double)))
   ("create_for_stream"
    (:args (write-func write-func-t) (closure :pointer)
	   (width-in-points :double) (height-in-points :double))))
  (:methods
   ("restrict_to_version" (:args (version svg-version-t)))))

(defcairofun "svg_get_versions"
  (:args (versions :pointer) (num-versions :pointer)))

(defun svg-get-versions* ()
  (with-foreign-objects ((versionsp :pointer) (num-versions :int))
    (svg-get-versions versionsp num-versions)
    (let ((versions (mem-ref versionsp :pointer)))
      (loop :for x :from 0 :below (mem-ref num-versions :int)
	    :collect (mem-aref versions 'svg-version-t x)))))

(export 'svg-get-versions*)

(defcairofun "svg_version_to_string"
  (:return-type :string)
  (:args (version svg-version-t)))

(defcairoclass "xlib_surface" surface surface-t
  (:methods
   ("set_size" (:args (width :int) (height :int)))
   ("get_width" (:return-type :int))
   ("get_height" (:return-type :int))
   ("get_depth" (:return-type :int))))