;;;
;;; Utils to deal with cairo internals and things that didn't fit elsewhere,
;;; as well as things that are categorized as utilities in cairo itself.
;;; See http://www.cairographics.org/manual/Support.html
;;;

(in-package :cffi-cairo)

(export '(version-encode version-decode deg-to-rad))

(defcairoclass "matrix" matrix matrix-pt
  (:methods
   ("init"
    (:args (xx :double) (yx :double) (xy :double) (yy :double)
	   (x0 :double) (y0 :double)))
   ("init_identity")
   ("init_translate" (:args (tx :double) (ty :double)))
   ("init_scale" (:args (sx :double) (sy :double)))
   ("init_rotate" (:args (radians :double)))
   ("translate" (:args (tx :double) (ty :double)))
   ("scale" (:args (sx :double) (sy :double)))
   ("rotate" (:args (radians :double)))
   ("invert")
   ("multiply" (:args (a matrix-pt) (b matrix-pt)))
   ("transform_distance"
    (:args (dx :pointer) (dy :pointer))
    (:alternative (:args dx dy)
		  (:pointers (dx :double) (dy :double))
		  (:body (list dx* dy*))))
   ("transform_point"
    (:args (x :pointer) (y :pointer))
    (:alternative (:args x y)
		  (:pointers (x :double) (y :double))
		  (:body (list x* y*))))))

(defcairofun "status_to_string"
  (:return-type :string)
  (:args (status status-t)))

(defcairofun "debug_reset_static_data")

(defcairofun "version" (:return-type :int))

(defcairofun "version_string" (:return-type :string))

(defun version-encode (major minor micro)
  (+ (* 10000 major) (* 100 minor) micro))

(defun version-decode (version)
  (multiple-value-bind (major rest) (floor version 10000)
    (multiple-value-bind (minor micro) (floor rest 100)
      (list major minor micro))))

(defun deg-to-rad (deg) (* deg (/ pi 180.0d0)))