;; ASDF load the cairo library
(asdf:oos 'asdf:load-op :cffi-cairo)

;; Load some crude sdl bindings
(load "sdl")

;; Make a test package
(defpackage :cairo-test
  (:use :common-lisp :cffi-cairo :cffi-sdl))

(in-package :cairo-test)

;; Set-up SDL - Tie sdl buffer memory to cairo output.
;; This relies on a common memory model of sdl and cairo
;; and may be a bit fragile.

(init +INIT-VIDEO+)

(defparameter *width* 640)
(defparameter *height* 480)
(defparameter *surface*
  (set-video-mode *width* *height* 32 cffi-sdl::+SWSURFACE+))
(defparameter *cairo-surface*
  (image-surface-create-for-data (get-pixels *surface*)
				 :cairo-format-argb32 *width* *height*
				 (get-pitch *surface*)))
(defparameter *cr* (create *cairo-surface*))

;; Update the screen every second
#+sbcl
(sb-thread:make-thread
 (lambda () (loop do (sleep 1)
		  do (unless (null *surface*) (flip *surface*))))
 :name "video-update")

(defparameter *my-pattern* (pattern-create-rgba 1d0 0.4d0 0.4d0 0.6d0))

;; Draw some stuff (basically taken from the cairo examples page).
(let ((xy-min (min *width* *height*)))
  (let ((cr *cr*)
	(xc (/ *width* 2d0)) (yc (/ *height* 2d0))
	(radius (* xy-min 0.4d0))
	(angle1 (deg-to-rad 45.0d0))
	(angle2 (deg-to-rad 180.0d0)))
    (set-line-width cr (* xy-min 0.01d0))
    (set-source cr *my-pattern*)
    (arc cr xc yc (* xy-min 0.1d0) 0d0 (* 2d0 pi))
    (fill-path cr)
    (set-line-width cr 5d0)
    (arc cr xc yc radius (deg-to-rad 44.9d0) angle1)
    (line-to cr xc yc)
    (arc cr xc yc radius (deg-to-rad 180.1d0) angle2)
    (line-to cr xc yc)
    (stroke cr)
    (set-source-rgba cr 1d0 1d0 1d0 1d0)
    (arc cr xc yc radius angle1 angle2)
    (stroke cr)
    (set-source-rgba cr 0d0 0d0 1d0 1d0)
    (arc-negative cr xc yc radius angle1 angle2)
    (stroke cr)))

;; Manipulate the transformation matrix
(scale *cr* 2d0 3d0)
(user-to-device* *cr* 10 20)

;; Write onto the screen
(select-font-face
 *cr* "Sans" :CAIRO-FONT-SLANT-NORMAL :CAIRO-FONT-WEIGHT-BOLD)
(set-font-size *cr* 10d0)
(move-to *cr* 10d0 10d0)
(show-text *cr* "Hello")
(show-text *cr* "Foo")

;; Various ways to update the screen
(fill-rect* *surface* 0 0 *width* *height* 0 0 0)
(flip *surface*)

