(defpackage :cffi-sdl
  (:use :common-lisp :cffi)
  (:export
   +INIT-TIMER+ +INIT-AUDIO+ +INIT-VIDEO+ +INIT-CDROM+ +INIT-JOYSTICK+
   +INIT-NOPARACHUTE+ +INIT-EVENTTHREAD+ +INIT-EVERYTHING+
   +SWSURFACE+ +HWSURFACE+ +ASYNCBLIT+ +ANYFORMAT+ +HWPALETTE+
   +DOUBLEBUF+ +FULLSCREEN+ +OPENGL+ +OPENGLBLIT+ +RESIZABLE+
   +NOFRAME+ +HWACCEL+ +SRCCOLORKEY+ +RLEACCELOK+ +RLEACCEL+
   +SRCALPHA+ +PREALLOC+
   init set-video-mode map-rgb update-rect flip fill-rect
   get-pixels get-pitch fill-rect*))

(in-package :cffi-sdl)

(define-foreign-library libsdl
  (:unix (:or "libSDL.so.11" "libSDL.so"))
  (t (:default "libSDL")))
   
(use-foreign-library libsdl)

(defconstant +INIT-TIMER+          #x00000001)
(defconstant +INIT-AUDIO+          #x00000010)
(defconstant +INIT-VIDEO+          #x00000020)
(defconstant +INIT-CDROM+          #x00000100)
(defconstant +INIT-JOYSTICK+       #x00000200)
(defconstant +INIT-NOPARACHUTE+    #x00100000)
(defconstant +INIT-EVENTTHREAD+    #x01000000)
(defconstant +INIT-EVERYTHING+     #x0000FFFF)

(defconstant +SWSURFACE+   #x00000000
  "Surface is in system memory")
(defconstant +HWSURFACE+   #x00000001
  "Surface is in video memory")
(defconstant +ASYNCBLIT+   #x00000004
  "Use asynchronous blits if possible")
(defconstant +ANYFORMAT+   #x10000000
  "Allow any video depth/pixel-format")
(defconstant +HWPALETTE+   #x20000000
  "Surface has exclusive palette")
(defconstant +DOUBLEBUF+   #x40000000
  "Set up double-buffered video mode")
(defconstant +FULLSCREEN+  #x80000000
  "Surface is a full screen display")
(defconstant +OPENGL+      #x00000002
  "Create an OpenGL rendering context")
(defconstant +OPENGLBLIT+  #x0000000A
  "Create an OpenGL rendering context and use it for blitting")
(defconstant +RESIZABLE+   #x00000010
  "This video mode may be resized")
(defconstant +NOFRAME+     #x00000020
  "No window caption or edge frame")
(defconstant +HWACCEL+     #x00000100
  "Blit uses hardware acceleration")
(defconstant +SRCCOLORKEY+ #x00001000
  "Blit uses a source color key")
(defconstant +RLEACCELOK+  #x00002000
  "Private flag")
(defconstant +RLEACCEL+    #x00004000
  "Surface is RLE encoded")
(defconstant +SRCALPHA+    #x00010000
  "Blit uses source alpha blending")
(defconstant +PREALLOC+    #x01000000
  "Surface uses preallocated memory")


(defcstruct rect-t
  (x :int16) (y :int16)
  (w :uint16) (h :uint16))

(defcstruct pixelformat-t
  (palette :pointer)
  (bits-per-pixel :uint8)
  (bytes-per-pixel :uint8)
  (r-loss :uint8)
  (g-loss :uint8)
  (b-loss :uint8)
  (a-loss :uint8)
  (r-shift :uint8)
  (g-shift :uint8)
  (b-shift :uint8)
  (a-shift :uint8)
  (r-mask :uint32)
  (g-mask :uint32)
  (b-mask :uint32)
  (a-mask :uint32)
  (colorkey :uint32)
  (alpha :uint8))

(defcstruct surface-t
  (flags :uint32)
  (format :pointer)
  (w :int) (h :int)
  (pitch :uint16)
  (pixels :pointer)
  (offset :int)
  (private-hwdata :pointer)
  (clip-rect rect-t)
  (unused1 :uint32)
  (locked :uint32)
  (map :pointer)
  (format-version :uint)
  (refcount :int))

(defcfun ("SDL_Init" init) :int (flags :uint32))
(defcfun ("SDL_SetVideoMode" set-video-mode)
    :pointer
  (width :int) (height :int) (bpp :int) (flags :uint32))
(defcfun ("SDL_MapRGB" map-rgb)
    :uint32
  (format pixelformat-t) (r :uint8) (g :uint8) (b :uint8))
(defcfun ("SDL_UpdateRect" update-rect)
    :void
  (surface :pointer) (x :int32) (y :int32) (w :uint32) (h :uint32))
(defcfun ("SDL_Flip" flip) :void (surface :pointer))
(defcfun ("SDL_FillRect" fill-rect)
    :int
  (surface :pointer) (rect :pointer) (color :uint32))


(defun get-pixels (surface)
  (foreign-slot-value surface 'surface-t 'pixels))
(defun get-pitch (surface)
  (foreign-slot-value surface 'surface-t 'pitch))
(defun fill-rect* (surface x y width height r g b)
  (let ((color (map-rgb (foreign-slot-value surface 'surface-t 'format) r g b)))
    (with-foreign-object (rect 'rect-t)
      (setf (foreign-slot-value rect 'rect-t 'x) x
	    (foreign-slot-value rect 'rect-t 'y) y
	    (foreign-slot-value rect 'rect-t 'w) width
	    (foreign-slot-value rect 'rect-t 'h) height)
      (fill-rect surface rect color))))